@@ -7,7 +7,7 @@
  * parallel execution.
  *
  * Written by Theodore Ts'o, <tytso@mit.edu>
- * 
+ *
  * Miquel van Smoorenburg (miquels@drinkel.ow.org) 20-Oct-1994:
  *   o Changed -t fstype to behave like with mount when -A (all file
  *     systems) or -M (like mount) is specified.
@@ -16,7 +16,7 @@
  *     can be added without changing this front-end.
  *   o -R flag skip root file system.
  *
- * Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 
+ * Copyright (C) 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000,
  * 	2001, 2002, 2003, 2004, 2005 by  Theodore Ts'o.
  *
  * %Begin-Header%
@@ -25,9 +25,11 @@
  * %End-Header%
  */
 
+#define _XOPEN_SOURCE 600 /* for inclusion of sa_handler in Solaris */
+
+#include "config.h"
 #include <sys/types.h>
 #include <sys/wait.h>
-#include <sys/signal.h>
 #include <sys/stat.h>
 #include <limits.h>
 #include <stdio.h>
@@ -231,18 +233,15 @@ static void parse_escape(char *word)
 
 static void free_instance(struct fsck_instance *i)
 {
-	if (i->prog)
-		free(i->prog);
-	if (i->device)
-		free(i->device);
-	if (i->base_device)
-		free(i->base_device);
+	free(i->prog);
+	free(i->device);
+	free(i->base_device);
 	free(i);
 	return;
 }
 
 static struct fs_info *create_fs_device(const char *device, const char *mntpnt,
-					const char *type, const char *opts, 
+					const char *type, const char *opts,
 					int freq, int passno)
 {
 	struct fs_info *fs;
@@ -308,9 +307,8 @@ static int parse_fstab_line(char *line, struct fs_info **ret_fs)
 	fs = create_fs_device(device, mntpnt, type ? type : "auto", opts,
 			      freq ? atoi(freq) : -1,
 			      passno ? atoi(passno) : -1);
-	if (dev)
-		free(dev);
-	   
+	free(dev);
+
 	if (!fs)
 		return -1;
 	*ret_fs = fs;
@@ -320,7 +318,7 @@ static int parse_fstab_line(char *line, struct fs_info **ret_fs)
 static void interpret_type(struct fs_info *fs)
 {
 	char	*t;
-	
+
 	if (strcmp(fs->type, "auto") != 0)
 		return;
 	t = blkid_get_tag_value(cache, "TYPE", fs->device);
@@ -363,21 +361,22 @@ static void load_fs_info(const char *filename)
 		else
 			old_fstab = 0;
 	}
-	
+
 	fclose(f);
-	
+
 	if (old_fstab && filesys_info) {
-		fputs(_("\007\007\007"
+		fputs("\007\007\007", stderr);
+		fputs(_(
 		"WARNING: Your /etc/fstab does not contain the fsck passno\n"
 		"	field.  I will kludge around things for you, but you\n"
 		"	should fix your /etc/fstab file as soon as you can.\n\n"), stderr);
-		
+
 		for (fs = filesys_info; fs; fs = fs->next) {
 			fs->passno = 1;
 		}
 	}
 }
-	
+
 /* Lookup filesys in /etc/fstab and return the corresponding entry. */
 static struct fs_info *lookup(char *filesys)
 {
@@ -449,7 +448,7 @@ static int execute(const char *type, const char *device, const char *mntpt,
 	sprintf(prog, "fsck.%s", type);
 	argv[0] = string_copy(prog);
 	argc = 1;
-	
+
 	for (i=0; i <num_args; i++)
 		argv[argc++] = string_copy(args[i]);
 
@@ -488,7 +487,7 @@ static int execute(const char *type, const char *device, const char *mntpt,
 			printf("%s ", argv[i]);
 		printf("\n");
 	}
-	
+
 	/* Fork and execute the correct program. */
 	if (noexecute)
 		pid = -1;
@@ -507,7 +506,7 @@ static int execute(const char *type, const char *device, const char *mntpt,
 
 	for (i=0; i < argc; i++)
 		free(argv[i]);
-	
+
 	inst->pid = pid;
 	inst->prog = string_copy(prog);
 	inst->type = string_copy(type);
@@ -525,7 +524,7 @@ static int execute(const char *type, const char *device, const char *mntpt,
 		p->next = inst;
 	else
 		instance_list = inst;
-	
+
 	return 0;
 }
 
@@ -578,7 +577,7 @@ static struct fsck_instance *wait_one(int flags)
 	 * (inst and prev are thought to be uninitialized variables)
 	 */
 	inst = prev = NULL;
-	
+
 	do {
 		pid = waitpid(-1, &status, flags);
 		if (cancel_requested && !kill_sent) {
@@ -607,7 +606,7 @@ static struct fsck_instance *wait_one(int flags)
 		}
 	} while (!inst);
 
-	if (WIFEXITED(status)) 
+	if (WIFEXITED(status))
 		status = WEXITSTATUS(status);
 	else if (WIFSIGNALED(status)) {
 		sig = WTERMSIG(status);
@@ -692,11 +691,11 @@ static int wait_many(int flags)
 
 /*
  * Run the fsck program on a particular device
- * 
+ *
  * If the type is specified using -t, and it isn't prefixed with "no"
  * (as in "noext2") and only one filesystem type is specified, then
  * use that type regardless of what is specified in /etc/fstab.
- * 
+ *
  * If the type isn't specified by the user, then use either the type
  * specified in /etc/fstab, or DEFAULT_FSTYPE.
  */
@@ -710,7 +709,7 @@ static void fsck_device(struct fs_info *fs, int interactive)
 	if (strcmp(fs->type, "auto") != 0)
 		type = fs->type;
 	else if (fstype && strncmp(fstype, "no", 2) &&
-	    strncmp(fstype, "opts=", 5) && strncmp(fstype, "loop", 4) && 
+	    strncmp(fstype, "opts=", 5) && strncmp(fstype, "loop", 4) &&
 	    !strchr(fstype, ','))
 		type = fstype;
 	else
@@ -759,7 +758,7 @@ static void compile_fs_type(char *fs_type, struct fs_type_compile *cmp)
 	cmp->list = malloc(num * sizeof(char *));
 	cmp->type = malloc(num * sizeof(int));
 	if (!cmp->list || !cmp->type) {
-		fputs(_("Couldn't allocate memory for filesystem types\n"), 
+		fputs(_("Couldn't allocate memory for filesystem types\n"),
 		      stderr);
 		exit(EXIT_ERROR);
 	}
@@ -769,7 +768,7 @@ static void compile_fs_type(char *fs_type, struct fs_type_compile *cmp)
 
 	if (!fs_type)
 		return;
-	
+
 	list = string_copy(fs_type);
 	num = 0;
 	s = strtok(list, ",");
@@ -820,7 +819,7 @@ static int opt_in_list(const char *opt, char *optlist)
 	if (!optlist)
 		return 0;
 	list = string_copy(optlist);
-	
+
 	s = strtok(list, ",");
 	while(s) {
 		if (strcmp(s, opt) == 0) {
@@ -894,7 +893,7 @@ static int ignore(struct fs_info *fs)
 	 * ignore it.
 	 */
 	if (!fs_match(fs, &fs_type_compiled)) return 1;
-	
+
 	/* Are we ignoring this type? */
 	for(ip = ignored_types; *ip; ip++)
 		if (strcmp(fs->type, *ip) == 0) return 1;
@@ -943,7 +942,7 @@ static int device_already_active(char *device)
 	 * If we don't know the base device, assume that the device is
 	 * already active if there are any fsck instances running.
 	 */
-	if (!base) 
+	if (!base)
 		return (instance_list != 0);
 	for (inst = instance_list; inst; inst = inst->next) {
 		if (!inst->base_device || !strcmp(base, inst->base_device)) {
@@ -976,7 +975,7 @@ static int check_all(NOARGS)
 		if (ignore(fs))
 			fs->flags |= FLAG_DONE;
 	}
-		
+
 	/*
 	 * Find and check the root filesystem.
 	 */
@@ -1060,7 +1059,7 @@ static int check_all(NOARGS)
 		status |= wait_many(pass_done ? FLAG_WAIT_ALL :
 				    FLAG_WAIT_ATLEAST_ONE);
 		if (pass_done) {
-			if (verbose > 1) 
+			if (verbose > 1)
 				printf("----------------------------------\n");
 			passno++;
 		} else
@@ -1105,7 +1104,7 @@ static void PRS(int argc, char *argv[])
 	sigaction(SIGINT, &sa, 0);
 	sigaction(SIGTERM, &sa, 0);
 #endif
-	
+
 	num_devices = 0;
 	num_args = 0;
 	instance_list = 0;
@@ -1175,14 +1174,14 @@ static void PRS(int argc, char *argv[])
 						progress_fd = 0;
 					else
 						goto next_arg;
-				} else if ((i+1) < argc && 
+				} else if ((i+1) < argc &&
 					   !strncmp(argv[i+1], "-", 1) == 0) {
 					progress_fd = string_to_int(argv[i]);
 					if (progress_fd < 0)
 						progress_fd = 0;
 					else {
+						++i;
 						goto next_arg;
-						i++;
 					}
 				}
 				break;
@@ -1293,7 +1292,7 @@ int main(int argc, char *argv[])
 	} else {
 		fsck_path = string_copy(fsck_prefix_path);
 	}
-	
+
 	if ((num_devices == 1) || (serialize))
 		interactive = 1;
 
@@ -1333,7 +1332,7 @@ int main(int argc, char *argv[])
 				status |= inst->exit_status;
 				free_instance(inst);
 			}
-			if (verbose > 1) 
+			if (verbose > 1)
 				printf("----------------------------------\n");
 		}
 	}
